-----------------------------------------------------------------------------------------------
-- Client Lua Script for ChargeSortedBank
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Apollo"
require "Window"
require "Money"
require "AccountItemLib"
require "StorefrontLib"



local knMaxBags = 10 -- how many max Bankslots there are
local kChargeSortedBagsDefaults = {
	char = {
		currentProfile = nil,
	},
	profile = {
		general = {
			optionsList = {
				General = {
						knSizeIconOption = 40,
						knBagsWidth = 654,
					},
       	 	Design = {
	        	fOpacity = 1,
	        	Main = 1,
	        	BG = 2,
	        	BGColor = "ffffffff",
        	},
				Thanks = {
				},
			},
			BagList = {
				Armor = {},
				Weapons = {},
				Consumables = {},
				Trash = {},
				Toys = {},
				Token = {},
				Crafting = {},
				Rest = {},
				Housing = {},
				Collectibles = {},
			},
			BagListName = {"Armor", "Weapons", "Consumables", "Trash", "Toys", "Token", "Crafting", "Rest", "Housing", "Collectibles"
			 },
			CustomBagListName = {},
			tAnchorOffsetBank = { 10, 10, 664, 416 },
			tAnchorOffsetOpt = {-401, -261, 401, 300 },
		},
	},
}


local Qualitys = {
	[1] = "BK3:UI_BK3_ItemQualityGrey",
	[2] = "BK3:UI_BK3_ItemQualityWhite",
	[3] = "BK3:UI_BK3_ItemQualityGreen",
	[4] = "BK3:UI_BK3_ItemQualityBlue",
	[5] = "BK3:UI_BK3_ItemQualityPurple",
	[6] = "BK3:UI_BK3_ItemQualityOrange",
	[7] = "BK3:UI_BK3_ItemQualityMagenta",
	}

local NumberQualitys = {
	[1] = "darkgray",
	[2] = "UI_WindowTextDefault",
	[3] = "AddonOk",
	[4] = "xkcdBrightBlue",
	[5] = "xkcdBarney",
	[6] = "Amber",
	[7] = "magenta",
	}

  local Designs = {
   [1] = "AbilitiesSprites:spr_StatVertProgBase",
   [2] = "CRB_Basekit:kitScrollbase_Horiz_HoloSmall",
   [3] = "CRB_DEMO_CreationSprites:sprDemo_CCFrameBG",
   [4] = "CRB_MegamapSprites:sprMap_WorldMap",
   [5] = "CRB_TooltipSprites:sprTT_BasicBack",
   [6] = "charactercreate:sprCharC_BG02",
  }


--compare Function
local fnSortItemsByName = function(Left, Right)
   local itemLeft = RealBag:GetItem(Left)
   local itemRight = RealBag:GetItem(Right)
	local strLeftName = itemLeft:GetName()
	local strRightName = itemRight:GetName()
	return strLeftName > strRightName
end

local fnSortItemsBylvl = function(itemLeft, itemRight)
	if itemLeft == itemRight then
		return 0
	end
	if itemLeft and itemRight == nil then
		return -1
	end
	if itemLeft == nil and itemRight then
		return 1
	end
	local leftilvl = itemLeft:GetDetailedInfo()["tPrimary"]["nItemLevel"]
	local rightilvl = itemRight:GetDetailedInfo()["tPrimary"]["nItemLevel"]
	if leftilvl < rightilvl then
		return -1
	end
	if leftilvl > rightilvl then
		return 1
	end

	return 0
end

local fnSortBagBySize = function(BagLeft, BagRight)
	local left = BagLeft:FindChild("BagGrid"):GetChildren()
	local right = BagLeft:FindChild("BagGrid"):GetChildren()

	return (#left) > (#right)
end
-----------------------------------------------------------------------------------------------
-- ChargeSortedBank Module Definition
-----------------------------------------------------------------------------------------------
local ChargeSortedBank = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function ChargeSortedBank:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function ChargeSortedBank:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- ChargeSortedBank OnLoad
-----------------------------------------------------------------------------------------------
function ChargeSortedBank:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("ChargeSortedBank.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	
	self.db = Apollo.GetPackage("Gemini:DB-1.0").tPackage:New(self, kChargeSortedBagsDefaults)
end

-----------------------------------------------------------------------------------------------
-- ChargeSortedBank OnDocLoaded
-----------------------------------------------------------------------------------------------
function ChargeSortedBank:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
		--Color Picker
	    GeminiColor = Apollo.GetPackage("GeminiColor").tPackage
		self.colorPicker = GeminiColor:CreateColorPicker(self, "ColorPickerCallback", false, "ffffffff")
	    self.colorPicker:Show(false, true)
			
		-- Events
			Apollo.RegisterEventHandler("HideBank", "OnCancel", self)
			Apollo.RegisterEventHandler("ShowBank", "Initialize", self)
   			Apollo.RegisterEventHandler("ToggleBank", "Initialize", self)
			Apollo.RegisterEventHandler("UpdateInventory", 	"OnUpdateInventory", self)
			Apollo.RegisterEventHandler("DragDropSysBegin",	"OnSystemBeginDragDrop", self)
			Apollo.RegisterEventHandler("DragDropSysEnd", "OnSystemEndDragDrop", self)
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		self.wndOptions =	Apollo.LoadForm(self.xmlDoc, "Options", nil, self)
		self.wndOptions:Show(false,true)



		-- Do additional Addon initialization here
		self.firstLoad = true
	end
end

function ChargeSortedBank:Initialize()
	if self.firstLoad then
		self.wndMain = 	Apollo.LoadForm(self.xmlDoc, "ChargeSortedBankForm", nil, self)
		self.wndNewBag = Apollo.LoadForm(self.xmlDoc, "NewBagForm", nil, self)
		self.OneSlot = self.wndMain:FindChild("OneBagSlot")
		self.RealBag = self.OneSlot:FindChild("RealBagWindow")	
		RealBag = self.OneSlot:FindChild("RealBagWindow")
		self.wndNewBag:Show(false,true)
		self:SetWindows()
		self:LoadSlots()
		self.firstLoad = false
		self:LoadBank()
		self:LoadGrid()
		self:ArrangeChildren()
	end
	self:RefreshStoreLink()
	self.wndMain:Show(true)
end

function ChargeSortedBank:LoadBank()
	for i,j in pairs(self.db.profile.general.BagListName) do
		self.db.profile.general.BagList[j] = {}
	end
	self.wndMain:FindChild("BagGrid"):DestroyChildren()
	local Bag = self.wndMain:FindChild("RealBagWindow")
	local maxSlots = Bag:GetTotalBagSlots()
	for i = 512, 512+maxSlots do
		local item = self.RealBag:GetItem(i)
		if item ~= nil then
			local isCustom = self:IsCustom(item)
			if not isCustom then
				local type = item:GetItemFamilyName()
				if type == "Gear" or type == "Armor" or type == "Costume" then
					table.insert(self.db.profile.general.BagList.Armor, i)
				elseif type == "Weapon" then
					table.insert(self.db.profile.general.BagList.Weapons, i)
				elseif type == "Junk" or type == "Miscellaneous" or type == "Trash" then
					if item:GetItemCategoryName()== "Toys" then
						table.insert(self.db.profile.general.BagList.Toys, i)
					elseif item:GetItemCategoryName() == "Loot Bag" or item:GetItemCategoryName() == "Salvageable Item" then
						table.insert(self.db.profile.general.BagList.Rest, i)
					else
						table.insert(self.db.profile.general.BagList.Trash, i)
					end
				elseif type == "Consumable" or type == "Imbuement Consumable" then
					table.insert(self.db.profile.general.BagList.Consumables, i)
				elseif type == "Housing" then
					table.insert(self.db.profile.general.BagList.Housing, i)
				elseif type == "Toys" then
					table.insert(self.db.profile.general.BagList.Toys, i)
				elseif type == "Token" then
					table.insert(self.db.profile.general.BagList.Token, i)
				elseif type == "Crafting" or type == "Rune" then
					table.insert(self.db.profile.general.BagList.Crafting, i)
				elseif type == "Collectible" then
					if item:GetItemTypeName() ==  "PvP Imbuement Material" then
						table.insert(self.db.profile.general.BagList.Consumables, i)
					else
						table.insert(self.db.profile.general.BagList.Collectibles, i)
					end
				else
					table.insert(self.db.profile.general.BagList.Rest, i)
				end
			end
		end
	end
	for i,j in pairs(self.db.profile.general.BagList) do
    if not self:IsCustomBag(i) then
		  self.db.profile.general.BagList[i] = self:MergeSortlvl(j)
		  self.db.profile.general.BagList[i] = self:NameSort(self.db.profile.general.BagList[i])
    end
	end

	self.BagCount = self.RealBag:GetTotalEmptyBagSlots()
	self.RealBag:MarkAllItemsAsSeen()
end

function ChargeSortedBank:LoadGrid()

	local BagGrid = self.wndMain:FindChild("BagGrid")
	BagGrid:DestroyChildren()
	for i,j in pairs(self.db["profile"]["general"]["BagListName"]) do
		local Bag = self.db["profile"]["general"]["BagList"][j]
		if Bag ~= nil and #Bag ~= 0 then
			local NewBag = Apollo.LoadForm(self.xmlDoc, "NewBag", BagGrid, self)
			NewBag:SetName(j)
			NewBag:FindChild("Title"):SetText(j)
			for l,k in ipairs(Bag) do
        		local item = self.RealBag:GetItem(k)
				local NewSlot = Apollo.LoadForm(self.xmlDoc, "BagItem", NewBag:FindChild("BagGrid"), self)
				NewSlot:SetAnchorOffsets(0, 0, self.db.profile.general.optionsList.General.knSizeIconOption, self.db.profile.general.optionsList.General.knSizeIconOption)
				NewSlot:SetName(l)
				NewSlot:FindChild("BagItemIcon"):SetSprite(item:GetIcon())--NewItem Sprite: BK3:UI_BK3_Holo_InsetSimple

				local details = item:GetDetailedInfo()["tPrimary"]

				--Stacks
				local count = details["tStack"]["nCount"]
				if count > 1 then
					NewSlot:FindChild("stack"):SetText(tostring(count))
				end
				--ilvl
				if j == "Armor" or j == "Weapons" then
					NewSlot:FindChild("ilvl"):SetText(tostring(details["nItemLevel"]))
				else
					NewSlot:FindChild("ilvl"):SetText("")
				end

				--Quality
				local quality = details["eQuality"]
				if quality ~= nil then
					NewSlot:SetSprite(Qualitys[quality])
					NewSlot:FindChild("ilvl"):SetTextColor(NumberQualitys[quality])
				end
			end
			NewBag:FindChild("BagGrid"):ArrangeChildrenTiles(0)
		end
	end


	for i,j in pairs(self.db["profile"]["general"]["CustomBagListName"]) do
		local Bag = self.db["profile"]["general"]["BagList"][j]
		if Bag ~= nil and #Bag ~= 0 then
			local NewBag = Apollo.LoadForm(self.xmlDoc, "NewBag", BagGrid, self)
			NewBag:SetName(j)
			NewBag:FindChild("Title"):SetText(j)
			NewBag:FindChild("Close"):Show(true)
			for l,k in ipairs(Bag) do
				local NewSlot = Apollo.LoadForm(self.xmlDoc, "BagItem", NewBag:FindChild("BagGrid"), self)
				NewSlot:SetAnchorOffsets(0, 0, self.db.profile.general.optionsList.General.knSizeIconOption, self.db.profile.general.optionsList.General.knSizeIconOption)
				NewSlot:SetName(l)
        local item = self.RealBag:GetItem(k)
        local type = item:GetItemFamilyName()
				NewSlot:FindChild("BagItemIcon"):SetSprite(item:GetIcon())--NewItem Sprite: BK3:UI_BK3_Holo_InsetSimple
				local details = item:GetDetailedInfo()["tPrimary"]

				--Stacks
				local count = details["tStack"]["nCount"]
				if count > 1 then
					NewSlot:FindChild("stack"):SetText(tostring(count))
				end
				--ilvl
				if type == "Weapon" or type == "Gear" or type == "Armor" or type == "Costume" then
					NewSlot:FindChild("ilvl"):SetText(tostring(details["nItemLevel"]))
				else
					NewSlot:FindChild("ilvl"):SetText("")
				end

				--Quality
				local quality = details["eQuality"]
				if quality ~= nil then
					NewSlot:SetSprite(Qualitys[quality])
					NewSlot:FindChild("ilvl"):SetTextColor(NumberQualitys[quality])
				end
			end
			NewBag:FindChild("BagGrid"):ArrangeChildrenTiles(0)
		else
			local NewBag = Apollo.LoadForm(self.xmlDoc, "NewBag", BagGrid, self)
			NewBag:SetName(j)
			NewBag:FindChild("Title"):SetText(j)
			local EmptySlot = Apollo.LoadForm(self.xmlDoc, "BagItem", NewBag:FindChild("BagGrid"), self)
			EmptySlot:SetName("1")
			EmptySlot:FindChild("Empty"):Show(true)
			EmptySlot:FindChild("ilvl"):SetText("")
			EmptySlot:SetSprite(Qualitys[1])
			NewBag:FindChild("Close"):Show(true)
			EmptySlot:SetAnchorOffsets(0, 0, self.db.profile.general.optionsList.General.knSizeIconOption, self.db.profile.general.optionsList.General.knSizeIconOption)
			NewBag:FindChild("BagGrid"):ArrangeChildrenTiles(0)
		end
	end
end

function ChargeSortedBank:OnUpdateInventory()
	if self.wndMain ~= nil and self.wndMain:IsShown() then
		self:LoadBank()
   		self:LoadSlots()
		local scroll = self.wndMain:FindChild("BagGrid"):GetVScrollPos()
		self:LoadGrid()
		self:ArrangeChildren()
		self.scroll = scroll
		self:RefreshStoreLink()
		self.RealBag:Show(false)
	end
end

function ChargeSortedBank:ArrangeChildren()
	local Options = self.db.profile.general.optionsList.General
	local titleHeight = 35
	local compensate = 16 --Since ArrangeChilderenTile() cuts off Tiles even before the max width is reached the Window needs to wider than columns*IconSize (therefor compensate)
	local BordersWidth = 61
	local smallBorder = Options.knSizeIconOption -10 -- The value needs to be smaller than the cube "-10" is best fitting
	local BagGrid = self.wndMain:FindChild("BagGrid")
	local Bags = BagGrid:GetChildren()

	table.sort(Bags, fnSortBagBySize)

	local a,b,c,d = self.wndMain:GetAnchorOffsets()
	local Width = c-a
	local LeftSlots = {}
	LeftSlots[1] = {0, 0 , 0 , 0}
	local counter = 1

	for i,j in pairs(Bags) do
		local Bag = j
		local slots = Bag:FindChild("BagGrid"):GetChildren()
		local slotsPerRow = math.floor((Options.knBagsWidth-BordersWidth)/Options.knSizeIconOption)
		if #slots >= slotsPerRow then
			local rows = math.ceil(#slots/slotsPerRow)
			counter = counter + 1
			LeftSlots[counter] = {LeftSlots[counter-1][1], LeftSlots[counter-1][4], LeftSlots[counter-1][1] + Options.knBagsWidth - BordersWidth + compensate,
				LeftSlots[counter-1][4]+ rows*Options.knSizeIconOption + titleHeight}
			Bag:SetAnchorOffsets( LeftSlots[counter-1][1], LeftSlots[counter-1][4], LeftSlots[counter-1][1] + Options.knBagsWidth - BordersWidth + compensate,
				LeftSlots[counter-1][4]+ rows*Options.knSizeIconOption + titleHeight)
		else
			local newHome = false
			for i,j in pairs(LeftSlots) do
				if i ~= 1 and newHome == false then
					local newWidth = j[3]+ #slots*Options.knSizeIconOption
					if (newWidth < Options.knBagsWidth-BordersWidth) then
						Bag:SetAnchorOffsets( j[3], j[2],newWidth+smallBorder,j[4])
						LeftSlots[i] = {j[1], j[2],newWidth+smallBorder,j[4]}
						newHome = true
					end
				end
			end
			if newHome == false then
				counter = counter +1
				local newWidth = LeftSlots[counter-1][1]+ #slots*Options.knSizeIconOption+smallBorder
				LeftSlots[counter] = {LeftSlots[counter-1][1],LeftSlots[counter-1][4], newWidth,LeftSlots[counter-1][4]+ Options.knSizeIconOption + titleHeight}
				Bag:SetAnchorOffsets(LeftSlots[counter-1][1], LeftSlots[counter-1][4], newWidth,LeftSlots[counter-1][4]+ Options.knSizeIconOption + titleHeight)
			end
		end
		Bag:FindChild("BagGrid"):ArrangeChildrenTiles(0)
	end
end


function ChargeSortedBank:LoadSlots()
  local Bag = self.wndMain:FindChild("RealBagWindow")
  local maxSlots = Bag:GetTotalBagSlots()
  local TextBox = self.wndMain:FindChild("Slots")

  --BagCount describes the empty Slot Count ;)
  self.BagCount = self.RealBag:GetTotalEmptyBagSlots()
	if self.RealBag:GetTotalEmptyBagSlots() == 0 then
		self.wndMain:FindChild("Border"):Show(true)
		self.wndMain:FindChild("Full"):SetText("full")
    TextBox:SetText("[ "..tostring(maxSlots).."/"..tostring(maxSlots).." ]")
    TextBox:SetTextColor("red")
  else
    self.wndMain:FindChild("Border"):Show(false)
		self.wndMain:FindChild("Full"):SetText("")
    if self.BagCount <= 5 then
      TextBox:SetText("[ "..tostring(maxSlots-self.BagCount).."/"..tostring(maxSlots).." ]")
      TextBox:SetTextColor("AttributeName")
    else
      TextBox:SetText("[ "..tostring(maxSlots-self.BagCount).."/"..tostring(maxSlots).." ]")
      TextBox:SetTextColor("cyan")
    end
	end
end

function ChargeSortedBank:RemoveItemFromCustomBag(item)
  local number = self:FindSlotNumber(item)
  local general = self.db.profile.general
  for i,j in pairs(general.CustomBagListName) do
    if general.BagList[j] ~= nil then
      for k,l in pairs(general.BagList[j]) do
          if l == number then
            table.remove(general.BagList[j],k)
            return true
          end
      end
    end
  end
  return false
end

function ChargeSortedBank:IsCustomBag(BagName)

  for i,j in pairs(self.db.profile.general.CustomBagListName) do
    if j == BagName then
      return true
    end
  end

  return fale
end

function ChargeSortedBank:IsCustom( item )

	local id = item:GetInventoryId()
	for i,j in pairs(self.db.profile.general.CustomBagListName) do
		if self.db.profile.general.BagList[j] ~= nil then
			for k,l in pairs(self.db.profile.general.BagList[j]) do
        local nextItem = self.RealBag:GetItem(l)
				if nextItem == nil then
					table.remove(self.db.profile.general.BagList[j],k)
				else
					if nextItem:GetInventoryId() == id then
						return true
					end
				end
			end
		end
	end
	return false
end



function ChargeSortedBank:OnCancel()
	self.wndMain:Close() -- hide the window
	Event_CancelBanking()
end

function ChargeSortedBank:OnGenerateTooltip( wndHandler, wndControl, eToolTipType, x, y )

	if wndHandler:FindChild("Empty"):IsShown() then
		return
	end

	if wndControl ~= wndHandler then return end
	wndControl:SetTooltipDoc(nil)
	local ItemNumber = tonumber(wndControl:GetParent():GetName())
	local list = wndControl:GetParent():GetParent():GetParent():FindChild("Title"):GetText()
	item =  self.RealBag:GetItem(self.db.profile.general.BagList[list][ItemNumber])
	if item ~= nil then
		local itemEquipped = item:GetEquippedItemForItemType()
		Tooltip.GetItemTooltipForm(self, wndControl, item, {bPrimary = true, bSelling = false, itemCompare = itemEquipped})
	end
end

function ChargeSortedBank:AddNewBagConfirm( wndHandler, wndControl)
	self.wndNewBag:Show(false)
	local BagName = wndControl:GetParent():FindChild("EditBox"):GetText()
	if BagName ~= "" and BagName ~= nil then
		BagName = self:NameOkTest(BagName)
		self.db.profile.general.BagList[BagName] = {}
		table.insert(self.db.profile.general.CustomBagListName,BagName)
	end
	self:LoadGrid()
	self:ArrangeChildren()
end

function ChargeSortedBank:NameOkTest(BagName)
	local Test = false
	local Number = 1
	local NewBagName = BagName
	while Test == false do
		local NoNameChange = true
		for i,j in pairs(self.db.profile.general.CustomBagListName) do
			if NewBagName == j then
				NewBagName = BagName.."("..tostring(Number)..")"
				Number = Number + 1
				NoNameChange = false
			end
		end
		if NoNameChange then
			return NewBagName
		end
	end
end

function ChargeSortedBank:AddNewBag( wndHandler, wndControl, eMouseButton )
	self.wndNewBag:Show(true)
	self.wndNewBag:FindChild("EditBox"):SetText("Name")
end


function ChargeSortedBank:OnDeleteCustomeBag( wndHandler, wndControl, eMouseButton )
	local BagName = wndControl:GetParent():FindChild("Title"):GetText()
	self.db.profile.general.BagList[BagName] = nil
	for i,j in pairs(self.db.profile.general.CustomBagListName) do
		if j == BagName then
			table.remove(self.db.profile.general.CustomBagListName, i)
			break
		end
	end
	wndControl:GetParent():Destroy()
 	self:LoadGrid()
	self:ArrangeChildren()
end

function ChargeSortedBank:OnBagItemMouseExit( wndHandler, wndControl, x, y )
  if self.OverSlot == wndHandler then
    self.RealBag:Show(false)
  end

  local Highl = wndControl:GetParent():FindChild("ItemHighlight")
  if Highl ~= nil then
	 Highl:Show(false)
  end
end


function ChargeSortedBank:OnBagItemMousEnter( wndHandler, wndControl, x, y )
	if wndHandler:FindChild("Empty"):IsShown() then
		return
	end
 	self.OverSlot = wndHandler
	--Highlight
	local Highlight = wndHandler:GetParent():FindChild("ItemHighlight")
	local NewItem = wndHandler:GetParent():FindChild("ItemNew")
	Highlight:Show(true)
	NewItem:Show(false)

	--Setting up BagWindow
	local OneSlot = self.wndMain:FindChild("OneBagSlot")
	local BagWindow = self.wndMain:FindChild("RealBagWindow")
	local maxSlots = BagWindow:GetBagCapacity()
	local iconSize = self.db.profile.general.optionsList.General.knSizeIconOption
	BagWindow:SetSquareSize(iconSize+1, iconSize+1)
	BagWindow:SetBoxesPerRow(maxSlots)

	local slot = tonumber(wndControl:GetParent():GetName())
	local Bag = wndControl:GetParent():GetParent()
	local BagName = Bag:GetParent():FindChild("Title"):GetText()
	local item = self.RealBag:GetItem(self.db.profile.general.BagList[BagName][slot])
  	if item ~= nil then
	  	local SlotNum = self:FindSlotNumber(item)-511
	  	local leftBorder, topBorder = self.wndMain:FindChild("BagGrid"):GetAnchorOffsets()-- The gab between wndMain and the BagGrid is 9  -- The gab between wndMain at the top is 44
	  	local itemgab =  math.floor(0.5*iconSize)-- idk why but it works
	  	local l,t,r,b = wndControl:GetParent():GetAnchorOffsets()
	  	local pl,pt,pr,pb = Bag:GetAnchorOffsets()
	  	local ppl,ppt, ppr,ppb = Bag:GetParent():GetAnchorOffsets()
	  	local scroll = self.wndMain:FindChild("BagGrid"):GetVScrollPos()
	  	OneSlot:SetAnchorOffsets(l+pl+ppl+leftBorder, t+pt+ppt+topBorder-scroll, l+pl+ppl+iconSize+leftBorder, t+pt+ppt+iconSize+topBorder-scroll)
	
	  	local offset =	SlotNum*-(iconSize+1)
	  	BagWindow:SetAnchorOffsets(offset+iconSize-itemgab,-1,(maxSlots)*(iconSize+1)+offset+iconSize-itemgab,iconSize-1)
	  	self.RealBag:Show(true)
    end
end

function ChargeSortedBank:FindSlotNumber(item)
	local BagWindow = self.wndMain:FindChild("RealBagWindow")
	local maxSlots = BagWindow:GetTotalBagSlots()
	local id = item:GetInventoryId()
	for i =512 , 512+maxSlots do
		local CurrItem = self.RealBag:GetItem(i)
		if CurrItem ~= nil and CurrItem:GetInventoryId() == id then
			return i
		end
	end
	return -1
end

function ChargeSortedBank:OnBagBtnMouseEnter( wndHandler, wndControl, x, y )
	local item = wndHandler:GetItem()
	wndControl:SetTooltipDoc(nil)
	Tooltip.GetItemTooltipForm(self, wndControl, item, {bPrimary = true, bSelling = false, itemCompare = nil})

end


function ChargeSortedBank:OnBagBtnMouseExit( wndHandler, wndControl, x, y )
	wndControl:SetTooltipDoc(nil)
end

---------------------------------------------------------------------------------------------------
-- Bags and Storefunktion
---------------------------------------------------------------------------------------------------

function ChargeSortedBank:UnlockMoreBankSlots()
	StorefrontLib.OpenLink(StorefrontLib.CodeEnumStoreLink.BankSlots)
end

function ChargeSortedBank:RefreshStoreLink()
	self.bStoreLinkValid = StorefrontLib.IsLinkValid(StorefrontLib.CodeEnumStoreLink.BankSlots)
	if self.wndMain and self.wndMain:IsValid() then
		self:Build()
	end
end

function ChargeSortedBank:Build()
	local nNumBagSlots = GameLib.GetNumBankBagSlots()

	self.wndMain:FindChild("ConfigureBagsContainer"):DestroyChildren()

	-- Configure Screen
	for idx = 1, GameLib.knMaxBankBagSlots do
		if not self.bStoreLinkValid and idx > nNumBagSlots then
			break
		end
		local idBag = idx + 20
		local wndCurr = Apollo.LoadForm(self.xmlDoc, "BankSlot", self.wndMain:FindChild("ConfigureBagsContainer"), self)
		local wndBagBtn = Apollo.LoadForm(self.xmlDoc, "BagBtn"..idBag, wndCurr:FindChild("BankSlotFrame"), self)
		if wndBagBtn:GetItem() then
			wndCurr:FindChild("BagCount"):SetText(wndBagBtn:GetItem():GetBagSlots())
		end
		wndCurr:FindChild("BagCount"):SetData(wndBagBtn)
		wndCurr:FindChild("NewBagPurchasedAlert"):Show(false, true)
		wndBagBtn:Enable(idx <= nNumBagSlots)
		
		if idx > nNumBagSlots + 1 then
			wndCurr:FindChild("BagLocked"):Show(true)
			wndCurr:SetTooltip(Apollo.GetString("Bank_LockedTooltip"))
		elseif idx > nNumBagSlots then
			wndCurr:FindChild("MTXUnlock"):Show(true)
			wndCurr:SetTooltip(Apollo.GetString("Bank_LockedTooltip"))
		else
			wndCurr:SetTooltip(Apollo.GetString("Bank_SlotTooltip"))
		end
	end
	self.wndMain:FindChild("ConfigureBagsContainer"):ArrangeChildrenHorz(Window.CodeEnumArrangeOrigin.Middle)
end

---------------------------------------------------------------------------------------------------
-- Utils
---------------------------------------------------------------------------------------------------

--all numbers are positive
function ChargeSortedBank:MergeSortlvl(tables)
	if #tables < 2 then
		return tables
	end

	local h = #tables/2
	local LTable = {}
	local RTable = {}
	for i = 1, #tables do
		if i <= h then
			table.insert(LTable,tables[i])
		else
			table.insert(RTable,tables[i])
		end
	end
	LTable = self:MergeSortlvl(LTable)
	RTable = self:MergeSortlvl(RTable)
	local max = #tables
	local Table = {}
	local left = 1
	local right = 1
	for i = 1, max do
		local leftilvl  = -1
		local rightilvl = -1
		if left < #LTable+1 then
			leftilvl  = self.RealBag:GetItem(LTable[left]):GetDetailedInfo()["tPrimary"]["nItemLevel"]
		end
		if right < #RTable+1 then
			rightilvl = self.RealBag:GetItem(RTable[right]):GetDetailedInfo()["tPrimary"]["nItemLevel"]
		end
		if leftilvl > rightilvl then
			Table[i] = LTable[left]
			left = left+1
		else
			Table[i] = RTable[right]
			right = right+1
		end
	end
	return Table
end


function ChargeSortedBank:NameSort(tables)
	if #tables == 0 then
		return
	end
	--the input is a ilvl-sorted table
	local All = {}
	local order = {}
	local t = 1
	for i = 1, #tables do
		local ilvl  = self.RealBag:GetItem(tables[i]):GetDetailedInfo()["tPrimary"]["nItemLevel"]
		if All[ilvl] == nil then
			All[ilvl] = {}
			order[t] = ilvl
			t= t+1
		end
		table.insert(All[ilvl],tables[i])
	end
	local returnTable = {}
	local z = 1;
	for i,j in pairs(order) do
		table.sort(All[j], fnSortItemsByName)
		for t,l in pairs(All[j]) do
			returnTable[z] = l
			z = z+1
		end
	end
	return returnTable
end


--thanks to Johan Lindström (Jabbit-EU, Joxye Nadrax / Wildstar) for the Split Function
-- Compatibility: Lua-5.1
function ChargeSortedBank:Split(str, pat)
   local t = {}  -- NOTE: use {n = 0} in Lua-5.0
   local fpat = "(.-)" .. pat
   local last_end = 1
   local s, e, cap = str:find(fpat, 1)
   while s do
      if s ~= 1 or cap ~= "" then
	 table.insert(t,cap)
      end
      last_end = e+1
      s, e, cap = str:find(fpat, last_end)
   end
   if last_end <= #str then
      cap = str:sub(last_end)
      table.insert(t, cap)
   end
   return t
end


-----------------------------------------------------------------------------------------------
-- ChargeSortedBank Instance
-----------------------------------------------------------------------------------------------
local ChargeSortedBankInst = ChargeSortedBank:new()
ChargeSortedBankInst:Init()
